import os, subprocess, shutil
from rdflib import Graph, BNode
from rdflib.namespace import RDFS
from rdflib import ConjunctiveGraph, URIRef, RDFS, RDF, Namespace
from reqman.apps.reqtool.rest_api.services.parse_ontologies import *
from reqman.apps.reqtool.models.main_req import SYSTEM_CHOICES, FUNCTION_CHOICES, INTERFACE_CHOICES, FLOW_CHOICES, ITEM_CHOICES, STATE_CHOICES, STATE_SET_CHOICES

#HERE
Ontology_file = "../../Ontologies/Autonomy_v1.ttl"
list_of_inference_metrics = ["IncompleteRequirement", "AmbiguousRequirement",
"InconsistentRequirement","NoisyRequirement","OpaqueRequirement","RedundantRequirement"]


main_syntax = get_main_sytax_inference()

def getclassofmain(main):
    return(findclassofmain(main))


def inferencing(project, boilerlate, prefix, main, suffix):
    metrics = {}
    listofboil = []
    listofprefix = []
    listofmain = []
    listofsuffix = []
    for i in boilerlate:
            listofboil.append(list(i.values()))
            #print(list(i.values('title'))[0]['title'])
            metrics[list(i.values('title'))[0]['title']] = []
    for i in prefix:
            listofprefix.append(list(i.values()))
    for i in main:
            listofmain.append(list(i.values()))
    for i in suffix:
            listofsuffix.append(list(i.values()))
    #findclassofmain(listofmain)
    ontology_with_new_req = exportboiltottl(project, listofprefix, listofboil, listofmain, listofsuffix)
    infer_result = shacl(ontology_with_new_req)
    #find in which metrics the requirements belongs e.g. title:REQ-1 is only an Incomplete req (and Ambiguous req), 
    # so -> 'REQ-1' : [Incomplete] ('REQ-1' : [Incomplete, Ambiguous])
    for met in infer_result:
        for title in metrics:
            if title in infer_result[met]:
                metrics[title].append(met)
    metrics.update(infer_result)
    return metrics


def exportboiltottl(project, prefix, boilerplate, main, suffix):
    ''' Export boilerplate to ttl file and return the file path'''
    #print(out_list) #A list with the values and the file that is the instance
    file_name = project.replace('/', '')
    if (os.path.exists("../../Ontologies/"+file_name+"/") == False):
        os.mkdir("../../Ontologies/"+file_name)
    save_path = "../../Ontologies/"+file_name+"/"
    completeName = os.path.join(save_path, file_name)
    per_instances = URIRef("http://delab.csd.auth.gr/ontologies/2018/RDO-instances#")
    rdo = URIRef("http://delab.csd.auth.gr/ontologies/2018/RDO#")
    rbo = URIRef("http://delab.csd.auth.gr/ontologies/2018/RBO#")
    lo = URIRef("http://delab.csd.auth.gr/ontologies/2018/LO-instances#")
    g = Graph()
    g.load(Ontology_file, format="turtle")
    #for b in boilerplate:
    for i in range(len(boilerplate)):
        title = boilerplate[i][0]['title']
        #title = BNode()
        g.add((per_instances + title, RDF.type, rdo + 'Requirement'))
        if (boilerplate[i][0]['has_prefix'] == True):
            g.add((per_instances + title, rdo + 'hasPrefix', per_instances+(title+"_"+ prefix[i][0]['simple_prefix'])))
        if (boilerplate[i][0]['has_main'] == True):
            g.add((per_instances + title, rdo + 'hasMain', per_instances+(title+"_"+ main[i][0]['main_choices'])))
            g.add((per_instances+(title+"_"+ main[i][0]['main_choices']), RDF.type, rbo + main[i][0]['main_choices']))
            if main[i][0]['sys_fun_inter'] != "":
                #need to know in which file is the SUbject, RDO instances? or RDO-AOCS instance? or somewhere else?
                for j in range(0,(len(out_list)-1),2):
                    if out_list[j]==main[i][0]['sys_fun_inter']:
                        spliter_1 = out_list[j].split(" : ")
                        instance_file = out_list[j+1] 
                        break
                g.add((per_instances+(title+"_"+ main[i][0]['main_choices']), rbo+"isRelatedToSubject", URIRef(str(instance_file))+"#"+ str(spliter_1[len(spliter_1) -1])))
            if main[i][0]['verb'] != "":
                mainverb = main[i][0]['verb'].split(':')
                g.add((per_instances+(title+"_"+ main[i][0]['main_choices']), rbo+"isRelatedToAction", lo + mainverb[1].strip()))
            if main[i][0]['flow_function_interface_item_system_state_stateset'] != "":
                for j in range(0,(len(out_list)-1),2):
                    if out_list[j]==main[i][0]['flow_function_interface_item_system_state_stateset']:
                        spliter_1 = out_list[j].split(" : ")
                        instance_file = out_list[j+1] 
                        break
                g.add((per_instances+(title+"_"+ main[i][0]['main_choices']), rbo+"isRelatedTo"+spliter_1[0], URIRef(str(instance_file))+"#"+ str(spliter_1[len(spliter_1) -1])))


        if (boilerplate[i][0]['has_suffix'] == True):
            g.add((per_instances + title, rdo + 'hasSuffix', per_instances+(title+"_"+ suffix[i][0]['suffix_choices'])))
        
        #if (boilerplate[i][0]['has_prefix'] == True):
         #   g.add((per_instances+(title+"_"+ main[i][0]['main_choices']), RDF.type, rbo + main[i][0]['main_choices']))
    g.serialize(destination=save_path+file_name+'.ttl', format='turtle')
    return (save_path+file_name)
    #print(g.serialize(format="turtle").decode("utf-8"))

def shacl(ontotlogy_file):
    metrics_dict = {}
    file1 = ontotlogy_file + ".ttl"
    target = ontotlogy_file+"infer.ttl"
    open(target, 'w').close()
    shutil.copyfile(file1, target)
    for i in range(3):
            print(i+1, 'Cycle of inferencing')
            result = subprocess.check_output(["./reqman/apps/reqtool/rest_api/services/shacl-1.3.2/bin/shaclinfer.sh", "-datafile", target])
            file_object = open(target, 'a')
            res = str(result,'utf-8')
            file_object.write(res.strip())
            file_object.close()
    
    #need to fiil with all the metrics
    g = Graph()
    g.load(target, format="turtle")
    for inf_metr in list_of_inference_metrics:
        metrics_dict[inf_metr] = []
        p = URIRef("http://delab.csd.auth.gr/ontologies/2018/RDO#"+inf_metr)
        for s, p, o in g.triples((None, RDF.type, p)):
            #metrics_dict["IncompleteRequirement"] = [s]
            get_title = str(s).split("#")
            metrics_dict[inf_metr].append(get_title[1])
    print(metrics_dict)
    return(metrics_dict)

def findclassofmain(listofmain):
    classlist = []
    for i in range(len(main_syntax)):
        verbsmain =  main_syntax[('M'+str(i+1))]['Verbs']
        subsmain =  main_syntax[('M'+str(i+1))]['Subject']
        attrsmain =  main_syntax[('M'+str(i+1))]['Attributes']
        #for currentmain in range(len(listofmain)):
        currentverb = listofmain['verb'].rsplit(' : ', 1)
        currentsubject = listofmain['sys_fun_inter']
        currentattr = listofmain['flow_function_interface_item_system_state_stateset']
        for verbmain in verbsmain:
            flag = 0
            if (verbmain.upper() == currentverb[len(currentverb)-1].upper() ):
                for submain in subsmain:
                    flag = 0
                    if submain == "System":
                        for sys in SYSTEM_CHOICES:
                            if sys[0] ==  currentsubject:
                                flag = 1
                                break
                    if submain == "Function":
                        for sys in FUNCTION_CHOICES:
                            if sys[0] ==  currentsubject:
                                flag = 1
                                break
                    if submain == "Interface":
                        for sys in INTERFACE_CHOICES:
                            if sys[0] ==  currentsubject:
                                flag = 1    
                                break
                    if flag == 1 :
                        for attrmain in attrsmain:
                            if attrmain == "Flow":
                                for sys in FLOW_CHOICES:
                                    if sys[0] ==  currentattr:
                                        classlist.append('M'+str(i+1))
                                        return(classlist)
                            if attrmain == "Item":
                                for sys in ITEM_CHOICES:
                                    if sys[0] ==  currentattr:
                                        classlist.append('M'+str(i+1))
                                        return(classlist)
                            if attrmain == "Function":
                                for sys in FUNCTION_CHOICES:
                                    if sys[0] ==  currentattr:
                                        classlist.append('M'+str(i+1))
                                        return(classlist)
                            if attrmain == "System":
                                for sys in SYSTEM_CHOICES:
                                    if sys[0] ==  currentattr:
                                        classlist.append('M'+str(i+1))
                                        return(classlist)
                            if attrmain == "Interface":
                                for sys in INTERFACE_CHOICES:
                                    if sys[0] ==  currentattr:
                                        classlist.append('M'+str(i+1))
                                        return(classlist)
                            if attrmain == "State":
                                for sys in (STATE_CHOICES):
                                    if sys[0] ==  currentattr:
                                        classlist.append('M'+str(i+1))
                                        return(classlist)
                            if attrmain == "StateSet":
                                for sys in (STATE_SET_CHOICES):
                                    if sys[0] ==  currentattr:
                                        classlist.append('M'+str(i+1))
                                        return(classlist)
            
    classlist.append("ERROR")
    return classlist
                              

