from django.urls import path, re_path
from django.conf.urls import url 


from .views import user_views, requirements_views, main_req_views, suffix_req_views

urlpatterns = [
    path('users/', user_views.UserList.as_view(), name='api-user-list'),
    path('users/<uuid:pk>/', user_views.UserDetail.as_view(), name='api-post-details'),
    path('groupboilerplates/', requirements_views.BoilerplateGroupCreateAPIView.as_view(), name='api-groupboilerplates-create'),
    path('groupboilerplates/<int:pk>/', requirements_views.BoilerplateGroupDetailsAPIView.as_view(), name='api-groupboilerplates-list'),

    path('groupboilerplates/<int:groupboil>/infer/', requirements_views.InferListAPIView.as_view(), name='api-infer-list'),
    path('groupboilerplates/<int:groupboil>/infer/<int:pk>/', requirements_views.InferDetailsAPIView.as_view(), name='api-infer-details'),
    
    path('groupboilerplates/<int:groupboil>/boilerplates/', requirements_views.BoilerplateListCreateAPIView.as_view(), name='api-requirements-list'),
    path('groupboilerplates/<int:groupboil>/boilerplates/<int:pk>/', requirements_views.BoilerplateDetailsAPIView.as_view(), name='api-requirements-details'),
    path('groupboilerplates/<int:groupboil>/boilerplates/boilerplatedata/', requirements_views.BoilerplateDataListAPIView.as_view(), name='api-boilerlplate-list-data'),
    re_path('groupboilerplates/<int:groupboil>/boilerplates/(?:title=(?P<title_1>\w+)/)?$', requirements_views.BoilerplateList.as_view(), name='api-requirements-details-per-title'), 
    
    #'''Prefix'''
    path('groupboilerplates/<int:groupboil>/boilerplates/<int:boilerplate>/prefix/', requirements_views.PrefixListAPIView.as_view(), name='api-prefix-list'),
    path('groupboilerplates/<int:groupboil>/boilerplates/<int:boilerplate>/prefix/<int:pk>/', requirements_views.PrefixDetailsAPIView.as_view(), name='api-prefix-details'),

    #'''MAIN'''
    path('groupboilerplates/<int:groupboil>/boilerplates/<int:boilerplate>/main/', main_req_views.MainListAPIView.as_view(), name='api-main-list'),
    path('groupboilerplates/<int:groupboil>/boilerplates/<int:boilerplate>/main/<int:pk>/', main_req_views.MainDetailsAPIView.as_view(), name='api-main-details'),
    #'''SUFFIX'''
    path('groupboilerplates/<int:groupboil>/boilerplates/<int:boilerplate>/suffix/', suffix_req_views.SuffixListAPIView.as_view(), name='api-suffix-list'),
    path('groupboilerplates/<int:groupboil>/boilerplates/<int:boilerplate>/suffix/<int:pk>/', suffix_req_views.SuffixDetailsAPIView.as_view(), name='api-suffix-details'),

    #PREFIX CHOICES DATA
    path('prefix/choices/simpleprefix/', requirements_views.PrefixChoicesViewSet.as_view(), name='simpleprefixchoices'),
    path('prefix/choices/logicalconnectivity/', requirements_views.LogConChoicesViewSet.as_view(), name='logicalconnectivitychoices'),
    path('prefix/choices/verbprefix/', requirements_views.VerbChoicesViewSet.as_view(), name='verbprefixchoices'),
    path('prefix/choices/sysfunitem/', requirements_views.FunItemSysChoicesViewSet.as_view(), name='sysfunitem'),
    path('prefix/choices/staflowfunitem/', requirements_views.StaFlowFunItemChoicesViewSet.as_view(), name='staflowfunitem'),

    #MAIN CHOICES DATA
    path('main/choices/subject/', main_req_views.SysFunIntChoicesViewSet.as_view(), name='mainsubject'),
    path('main/choices/stateitem/', main_req_views.StatItemChoicesViewSet.as_view(), name='mainstateitem'),
    path('main/choices/statevalue/', main_req_views.StatevalueChoicesViewSet.as_view(), name='mainstatevalue'),
    path('main/choices/shall/', main_req_views.ShallChoicesViewSet.as_view(), name='mainshall'),
    path('main/choices/verb/', main_req_views.VerbChoicesViewSet.as_view(), name='mainverb'),
    path('main/choices/quantifier/', main_req_views.QuantChoicesViewSet.as_view(), name='mainquantifier'),
    path('main/choices/numunit/', main_req_views.NumUnitChoicesViewSet.as_view(), name='mainnumunit'),
    path('main/choices/fifuintsysstatset/', main_req_views.FlFuIntItSysStatSetChoicesViewSet.as_view(), name='mainfifuintsysstatset'),
    path('main/choices/stsysconset/', main_req_views.StaSysConSetChoicesViewSet.as_view(), name='mainstsysconset'),

    #SUFFIX CHOICES DATA
    path('suffix/choices/suffix/', suffix_req_views.SuffixChoicesViewSet.as_view(), name='suffixsuffix'),
    path('suffix/choices/numunits/', suffix_req_views.NumChoicesViewSet.as_view(), name='suffixnum'),
    path('suffix/choices/flow/', suffix_req_views.FlowChoicesViewSet.as_view(), name='suffixflow'),

    #CLASSES System
    path('class/system', main_req_views.SystemClassViewSet.as_view(), name='class_system'),
    path('class/system/subclasses', main_req_views.SystemSubclasesViewSet.as_view(), name='subclass_system'),
    path('class/system/instance', main_req_views.SystemInstancesViewSet.as_view(), name='instances_of_system'),

    #CLASSES Function
    path('class/function', main_req_views.FunctionClassViewSet.as_view(), name='class_function'),
    path('class/function/subclasses', main_req_views.FunctionSubclasesViewSet.as_view(), name='subclass_function'),
    path('class/function/instance', main_req_views.FunctionInstancesViewSet.as_view(), name='instances_of_function'),

    #CLASSES Item
    path('class/item', main_req_views.ItemClassViewSet.as_view(), name='class_item'),
    path('class/item/subclasses', main_req_views.ItemSubclasesViewSet.as_view(), name='subclass_item'),
    path('class/item/instance', main_req_views.ItemInstancesViewSet.as_view(), name='instances_of_item'),

    #CLASSES Interface
    path('class/interface', main_req_views.InterfaceClassViewSet.as_view(), name='class_interface'),
    path('class/interface/subclasses', main_req_views.InterfaceSubclasesViewSet.as_view(), name='subclass_interface'),
    path('class/interface/instance', main_req_views.InterfaceInstancesViewSet.as_view(), name='instances_of_interface'),

    #CLASSES Connection
    path('class/connection', main_req_views.ConnectionClassViewSet.as_view(), name='class_connection'),
    path('class/connection/subclasses', main_req_views.ConnectionSubclasesViewSet.as_view(), name='subclass_connection'),
    path('class/connection/instance', main_req_views.ConnectionInstancesViewSet.as_view(), name='instances_of_connection'),

    #CLASSES Flow
    path('class/flow', main_req_views.FlowClassViewSet.as_view(), name='class_flow'),
    path('class/flow/subclasses', main_req_views.FlowSubclasesViewSet.as_view(), name='subclass_flow'),
    path('class/flow/instance', main_req_views.FlowInstancesViewSet.as_view(), name='instances_of_flow'),

    #CLASSES State
    path('class/state', main_req_views.StateClassViewSet.as_view(), name='class_state'),
    path('class/state/subclasses', main_req_views.StateSubclasesViewSet.as_view(), name='subclass_state'),
    path('class/state/instance', main_req_views.StateInstancesViewSet.as_view(), name='instances_of_state'),
    
    #Dictionary
    path('dictionary/', main_req_views.DictionaryViewSet.as_view(), name='dictionary'),
    path('mainsyntax/', main_req_views.MainSyntaxViewSet.as_view(), name='mainsyntax'),

]