from rest_framework.generics import ListAPIView, RetrieveUpdateDestroyAPIView, ListAPIView
from rest_framework import permissions

from django.http import HttpResponse, JsonResponse
from django.views.decorators.csrf import csrf_exempt
from rest_framework.parsers import JSONParser

from reqman.apps.reqtool.models.main_req import Main
from reqman.apps.reqtool.rest_api.serializers.main_req import MainSerializer

from reqman.apps.permissions import IsOwnerOrReadOnly
from reqman.apps.reqtool.rest_api.services import fuseki, inference
from reqman.apps.reqtool.rest_api.services.parse_ontologies import Dictionary_Data_list

from rest_framework.response import Response
from rest_framework.views import APIView

from reqman.apps.reqtool.models.main_req import SYSTEM_CHOICES, FUNCTION_CHOICES, INTERFACE_CHOICES, STATE_CHOICES, ITEM_CHOICES
from reqman.apps.reqtool.models.main_req import STATE_VALUE_CHOICES, SHALL_CHOICES, VERB_CHOICES, QUANTIFIER_CHOICES, NUMBER_UNITS_CHOICES
from reqman.apps.reqtool.models.main_req import FLOW_CHOICES, CONNECTION_CHOICES, STATE_SET_CHOICES

from reqman.apps.reqtool.models.main_req import SYSTEM_CLASSES_COMMENTS, SYSTEM_SUBCLASSES_COMMENTS, SYSTEM_INSTANCES_COMMENTS 
from reqman.apps.reqtool.models.main_req import FUNCTION_CLASSES_COMMENTS, FUNCTION_SUBCLASSES_COMMENTS, FUNCTION_INSTANCES_COMMENTS
from reqman.apps.reqtool.models.main_req import ITEM_CLASSES_COMMENTS, ITEM_SUBCLASSES_COMMENTS, ITEM_INSTANCES_COMMENTS
from reqman.apps.reqtool.models.main_req import INTERFACE_CLASSES_COMMENTS, INTERFACE_SUBCLASSES_COMMENTS, INTERFACE_INSTANCES_COMMENTS
from reqman.apps.reqtool.models.main_req import CONNECTION_CLASSES_COMMENTS, CONNECTION_SUBCLASSES_COMMENTS, CONNECTION_INSTANCES_COMMENTS
from reqman.apps.reqtool.models.main_req import FLOW_CLASSES_COMMENTS, FLOW_SUBCLASSES_COMMENTS, FLOW_INSTANCES_COMMENTS
from reqman.apps.reqtool.models.main_req import STATE_CLASSES_COMMENTS, STATE_SUBCLASSES_COMMENTS, STATE_INSTANCES_COMMENTS, MAIN_SYNTAX

from reqman.apps.reqtool.models.requirements import BoilerplateData
from rest_framework.exceptions import APIException



class MainListAPIView(ListAPIView):
    """
    API view to retrieve list of posts or create new
    """
    permission_classes = [permissions.IsAuthenticatedOrReadOnly]
    serializer_class = MainSerializer
    #
    queryset = Main.objects.all()

    def get_queryset(self):
        group_pk = self.kwargs['boilerplate']
        #print(group_pk)
        queryset_main = Main.objects.filter(boilerplate_of_main = group_pk)
        return queryset_main

    #Custom actions when POST
    def perform_create(self, serializer):
        serializer.save()
        #print the data of the post
        #write_fuseki = fuseki.FusekiActions()
        #write_fuseki.write(serializer.data)

class MainDetailsAPIView(RetrieveUpdateDestroyAPIView):
    """
    API view to retrieve, update or delete post
    """
    permission_classes = [permissions.IsAuthenticatedOrReadOnly]

    serializer_class = MainSerializer
    
    def get_queryset(self):
        group_pk = self.kwargs['boilerplate']
        #print(group_pk)
        queryset_main= Main.objects.filter(boilerplate_of_main = group_pk)
        return queryset_main

    #Custom actions when PUT
    def perform_update(self, serializer):
        instance = serializer.validated_data
        #add data in thn Model BoilerplateData
        es_instance = instance['sys_fun_inter'].rsplit(':', 1)
        ev_instance =instance['verb'].rsplit(' : ', 1)
        eo_instance = instance['flow_function_interface_item_system_state_stateset'].rsplit(':', 1)
        mn = (es_instance[len(es_instance)-1] + ' ' + instance['shall'] + ' ' + ev_instance[len(ev_instance)-1] + ' ' + eo_instance[len(eo_instance)-1])
        curmain_choices = inference.getclassofmain(instance)
        #instance['main_choices'] = curmain_choices[0]
        if curmain_choices[0] == 'ERROR':
            raise APIException("Main choices not in [M1-M16]")
        else:
            BoilerplateData.objects.filter(owner_data=instance['boilerplate_of_main'].owner, boilerplate_data_id = instance['boilerplate_of_main']).update(main_data = mn )
            instance = serializer.save(main_choices = curmain_choices[0])
        #instance = serializer.save()



    #Custom actions when DELETE
    def perform_destroy(self, instance):
        #instance = serializer.validated_data
        instance.delete()


class SysFunIntChoicesViewSet(APIView):

    def get(self, request):
        response = Response(SYSTEM_CHOICES + FUNCTION_CHOICES + INTERFACE_CHOICES)
        return response


class StatItemChoicesViewSet(APIView):

    def get(self, request):
        response = Response(STATE_CHOICES + ITEM_CHOICES)
        return response


class StatevalueChoicesViewSet(APIView):

    def get(self, request):
        response = Response(STATE_VALUE_CHOICES)
        return response


class ShallChoicesViewSet(APIView):

    def get(self, request):
        response = Response(SHALL_CHOICES)
        return response


class VerbChoicesViewSet(APIView):

    def get(self, request):
        response = Response(VERB_CHOICES)
        return response


class QuantChoicesViewSet(APIView):

    def get(self, request):
        response = Response(QUANTIFIER_CHOICES)
        return response


class NumUnitChoicesViewSet(APIView):

    def get(self, request):
        response = Response(NUMBER_UNITS_CHOICES)
        return response


class FlFuIntItSysStatSetChoicesViewSet(APIView):

    def get(self, request):
        response = Response(FLOW_CHOICES + FUNCTION_CHOICES + INTERFACE_CHOICES + ITEM_CHOICES + SYSTEM_CHOICES +STATE_CHOICES + STATE_SET_CHOICES)
        return response


class StaSysConSetChoicesViewSet(APIView):

    def get(self, request):
        response = Response(STATE_VALUE_CHOICES + SYSTEM_CHOICES + CONNECTION_CHOICES + STATE_SET_CHOICES)
        return response

class SystemClassViewSet(APIView):

    def get(self, request):
        response = Response(SYSTEM_CLASSES_COMMENTS)
        return response

class SystemSubclasesViewSet(APIView):

    def get(self, request):
        response = Response(SYSTEM_SUBCLASSES_COMMENTS)
        return response

class SystemInstancesViewSet(APIView):

    def get(self, request):
        response = Response(SYSTEM_INSTANCES_COMMENTS)
        return response

class FunctionClassViewSet(APIView):

    def get(self, request):
        response = Response(FUNCTION_CLASSES_COMMENTS)
        return response

class FunctionSubclasesViewSet(APIView):

    def get(self, request):
        response = Response(FUNCTION_SUBCLASSES_COMMENTS)
        return response

class FunctionInstancesViewSet(APIView):

    def get(self, request):
        response = Response(FUNCTION_INSTANCES_COMMENTS)
        return response

class ItemClassViewSet(APIView):

    def get(self, request):
        response = Response(ITEM_CLASSES_COMMENTS)
        return response

class ItemSubclasesViewSet(APIView):

    def get(self, request):
        response = Response(ITEM_SUBCLASSES_COMMENTS)
        return response

class ItemInstancesViewSet(APIView):

    def get(self, request):
        response = Response(ITEM_INSTANCES_COMMENTS)
        return response

class InterfaceClassViewSet(APIView):

    def get(self, request):
        response = Response(INTERFACE_CLASSES_COMMENTS)
        return response

class InterfaceSubclasesViewSet(APIView):

    def get(self, request):
        response = Response(INTERFACE_SUBCLASSES_COMMENTS)
        return response

class InterfaceInstancesViewSet(APIView):

    def get(self, request):
        response = Response(INTERFACE_INSTANCES_COMMENTS)
        return response

class ConnectionClassViewSet(APIView):

    def get(self, request):
        response = Response(CONNECTION_CLASSES_COMMENTS)
        return response

class ConnectionSubclasesViewSet(APIView):

    def get(self, request):
        response = Response(CONNECTION_SUBCLASSES_COMMENTS)
        return response

class ConnectionInstancesViewSet(APIView):

    def get(self, request):
        response = Response(CONNECTION_INSTANCES_COMMENTS)
        return response

class FlowClassViewSet(APIView):

    def get(self, request):
        response = Response(FLOW_CLASSES_COMMENTS)
        return response

class FlowSubclasesViewSet(APIView):

    def get(self, request):
        response = Response(FLOW_SUBCLASSES_COMMENTS)
        return response

class FlowInstancesViewSet(APIView):

    def get(self, request):
        response = Response(FLOW_INSTANCES_COMMENTS)
        return response

class StateClassViewSet(APIView):

    def get(self, request):
        response = Response(STATE_CLASSES_COMMENTS)
        return response

class StateSubclasesViewSet(APIView):

    def get(self, request):
        response = Response(STATE_SUBCLASSES_COMMENTS)
        return response

class StateInstancesViewSet(APIView):

    def get(self, request):
        response = Response(STATE_INSTANCES_COMMENTS)
        return response

class DictionaryViewSet(APIView):

    def get(self, request):
        response = Response(Dictionary_Data_list)
        return response

class MainSyntaxViewSet(APIView):

    def get(self, request):
        response = Response(MAIN_SYNTAX)
        return response