from rest_framework.generics import ListCreateAPIView, RetrieveUpdateDestroyAPIView, ListAPIView
from rest_framework import permissions

from django.http import HttpResponse, JsonResponse
from django.views.decorators.csrf import csrf_exempt
from rest_framework.parsers import JSONParser

from reqman.apps.reqtool.models.suffix_req import Suffix
from reqman.apps.reqtool.rest_api.serializers.suffix_req import SuffixSerializer

from reqman.apps.permissions import IsOwnerOrReadOnly
from reqman.apps.reqtool.rest_api.services import fuseki

from rest_framework.response import Response
from rest_framework.views import APIView

from reqman.apps.reqtool.models.suffix_req import S_CHOICES, NUMBER_UNITS_CHOICES, TIME_UNITS_CHOICES, FLOW_CHOICES
from reqman.apps.reqtool.models.suffix_req import NUMERICAL_AFFIRMATIVE_CHOICES, S2_CHOICES, S3_CHOICES, S4_CHOICES, S5_6_CHOICES
from reqman.apps.reqtool.models.requirements import BoilerplateData


class SuffixListAPIView(ListAPIView):
    """
    API view to retrieve list of posts or create new
    """
    permission_classes = [permissions.IsAuthenticatedOrReadOnly]
    serializer_class = SuffixSerializer
    #
    queryset = Suffix.objects.all()

    def get_queryset(self):
        group_pk = self.kwargs['boilerplate']
        #print(group_pk)
        queryset_suffix = Suffix.objects.filter(boilerplate_of_suffix = group_pk)
        return queryset_suffix

    #Custom actions when POST
    def perform_create(self, serializer):
        serializer.save()
        #print the data of the post
        #write_fuseki = fuseki.FusekiActions()
        #write_fuseki.write(serializer.data)

class SuffixDetailsAPIView(RetrieveUpdateDestroyAPIView):
    """
    API view to retrieve, update or delete post
    """
    permission_classes = [permissions.IsAuthenticatedOrReadOnly]

    serializer_class = SuffixSerializer
    
    def get_queryset(self):
        group_pk = self.kwargs['boilerplate']
        #print(group_pk)
        queryset_suffix = Suffix.objects.filter(boilerplate_of_suffix = group_pk)
        return queryset_suffix

    #Custom actions when PUT
    def perform_update(self, serializer):
        instance = serializer.validated_data
        #add data in thn Model BoilerplateData
        eo_instance = instance['flow'].rsplit(':', 1)
        sf = (instance['s_choices'] + ' ' + str(instance['numerical']) + ' ' + instance['mumerical_units'] + ' ' + eo_instance[len(eo_instance)-1])
        BoilerplateData.objects.filter(owner_data=instance['boilerplate_of_suffix'].owner, boilerplate_data_id = instance['boilerplate_of_suffix']).update(suffix_data = sf )
        ssuffix_choice = self.getclassofsuffix(instance["s_choices"])
        serializer.save(suffix_choices = ssuffix_choice)
        #print(self.new.changed_data)
        #instance = serializer.save()
        #write_fuseki = fuseki.FusekiActions()
        #write_fuseki.write(serializer.data)

    #Custom actions when DELETE
    def perform_destroy(self, instance):
        instance = serializer.validated_data
        #add data in thn Model BoilerplateData
        #print("deleted")
        instance.delete()
    
    def getclassofsuffix(self, suffix):
        for sys in NUMERICAL_AFFIRMATIVE_CHOICES:
            if sys[0] ==  suffix:
                return ("S1")
        for sys in S2_CHOICES:
            if sys[0] ==  suffix:
                return ("S2")
        for sys in S3_CHOICES:
            if sys[0] ==  suffix:
                return ("S3")
        for sys in S4_CHOICES:
            if sys[0] ==  suffix:
                return ("S4")
        #special case only one option for S5, S6
        if S5_6_CHOICES[0][0] ==  suffix:
            return ("S5")
        if S5_6_CHOICES[1][0] ==  suffix:
            return ("S6")


class SuffixChoicesViewSet(APIView):

    def get(self, request):
        response = Response(S_CHOICES)
        return response


class NumChoicesViewSet(APIView):

    def get(self, request):
        response = Response(NUMBER_UNITS_CHOICES+TIME_UNITS_CHOICES)
        return response


class FlowChoicesViewSet(APIView):

    def get(self, request):
        response = Response(FLOW_CHOICES)
        return response

